--[[
DHUD4_Abilities.lua $Rev: 22 $
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2010 by Horacio Hoyos

This file is part of DHUD4.

    DHUD4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD4.  If not, see <http://www.gnu.org/licenses/>.
]]
local DHUD4 = LibStub("AceAddon-3.0"):GetAddon("DHUD4")
local L = LibStub("AceLocale-3.0"):GetLocale("DHUD4")

local string_find = string.find
local _G = _G

local UPDATE_INTERVAL = 1

local Abilities = {
    defaults = {
        profile = {
            side = "l",
            scale = 1,
            fontSize = 10,
            tips = true,
            vehiCombos = true,
            colors = {
                border = {
                    f = {r=1,g=1,b=1},
                    h = {r=255/255,g=1125/255,b=125/255},
                    l = {r=255/255,g=165/255,b=0}
                },
            },
        },
    },
}
DHUD4.Abilities = Abilities

local visibleBars = {
}

local colors
local options
function Abilities:GetOptions()
	if not options then
        --Options table
    	options = {
    		type = "group",
			name = L["Abilities Module"],
            childGroups = "tab",
			args = {
                header = {
                    order = 1,
                    type = 'description',
                    name = L["The abilities module manages specific class abilities tracking."],
                },
                enabled = {
                    order = 2,
                    type = "toggle",
                    name = L["Enable Abilities"],
                    get = function() return DHUD4:GetModuleEnabled(MODNAME) end,
                    set = function(info, value) DHUD4:SetModuleEnabled(MODNAME, value) end,
                },
				general = {
					order = 3,
					type = 'group',
					name = L["General"],
					args = {
						desc = {
							order = 0,
							type = "description",
							name = L["Configure general functionality"],
						},
						side = {
							order = 1,
							type = 'select',
							name = L["Abilities Side"],
							desc = L["This setting will only be available if the Auras Module is disabled"],
							values = {["r"] = L["Right"], ["l"] = L["Left"]},
							disabled = function() return DHUD4:GetModuleEnabled("DHUD4_Auras"); end,
						},
						tips = {
							order = 2,
							type = 'toggle',
							name = L["Show Abilities tips"],
						},
						fontSize = {
							order = 3,
							type = 'range',
							name = L["Font size"],
							min = 6,
							max = 25,
							step = 1,
						},
                        scale = {
							order = 4,
							type = "range",
							name = L["Set Abilities' scale"],
							min = 0.5,
							max = 1,
							step = 0.05,
						},
						colors = {
							order = 5,
							type = 'group',
							name = L["Border Color"],
                            inline = true,
							args = {
								f = {
									type = 'color',
									name = L["Unavailable"],
									order = 1,
								},
								h = {
									type = 'color',
									name = L["Half Time"],
									order = 2,
								},
								l = {
									type = 'color',
									name = L["Ready"],
									order = 3,
								},
							},
						},

					},
				},
            },
		}
	end
	return options
end

-- On update event for abilities
local function OnUpdateAura(self, elapsed)

    self.auraTimer = self.auraTimer + elapsed
    if self.auraTimer >= UPDATE_INTERVAL then
        local timeLeft = 0;
        self.left = self.left - self.auraTimer;
        if self.left <= 0 then
            self.id = 0
            self.hasAura = false
            self:SetScript("OnUpdate", nil)
            self:SetScript("OnEnter", nil)
            return self:Hide()
        end
        self.auraTimer = 0
        if (self.timer) then
            frameText = _G[self:GetName().."Text"]
            frameText:SetText(DHUD4:FormatTime(self.left));
        end
        Abilities:Colorize(self.id)
        local w = frameText:GetStringWidth() + 10;
        frameText:SetWidth(w);
        if ( GameTooltip:IsOwned(self) ) then
            GameTooltip:SetUnitAura("target", self.id, self.filter);
        end
    end
end


function Abilities:PlaceAbilitiesFrame()

    local b3, b4, b5 = "f"..self.side.."3", "f"..self.side.."4", "f"..self.side.."5"
    local dis = 186 + math.max((visibleBars[b3] and 15 or 0), (visibleBars[b4] and 30 or 0),  (visibleBars[b5] and 45 or 0)) + DHUD4:GetFrameSpacing()
    --dis = dis/self.scale + 25*self.scale
    local anchorPoint = "RIGHT"
    if (self.side == "l") then
        dis = dis * -1
        anchorPoint = "LEFT"
    end
    self.parent:ClearAllPoints()
    self.parent:SetPoint(anchorPoint, DHUD4_MainFrame, "CENTER", dis, 0)
end

function Abilities:_Refresh()

    if (self.parent) then
        self.parent:Hide()
    end
    local newParentFrame = (self.side == "r") and DHUD4_RightAbilities or DHUD4_LeftAbilities
    if (not newParentFrame) then
        local name = (self.side == "r") and "DHUD4_RightAbilities" or "DHUD4_LeftAbilities"
        newParentFrame = CreateFrame("Frame", name, DHUD4_MainFrame)
        newParentFrame:SetHeight(256)
        newParentFrame:SetWidth(128)
    end
    for i = 1, 12 do
        local frameName = "DHUD4_A"..i
        local frame = _G[frameName]
        local frameCooldown = _G[frameName.."Cooldown"]
        if (frame) then
            frame:SetScale(self.scale)
            if (self.parent ~= newParentFrame) then
                frame:SetParent(newParentFrame)
            end
        end
        if ( frameCooldown ) then
            frameCooldown:SetFont(DHUD4:GetFont(), self.fontSize * DHUD4.GetScale())
        end
    end
    self.parent = newParentFrame
    if DHUD4:GetModuleEnabled("DHUD4_Player") then
        local player = DHUD4:GetModule("DHUD4_Player")
        local health, power = player:GetBars()
        visibleBars[health] = true
        visibleBars[power] = true
    end
    self:PlaceAbilitiesFrame()
    self.parent:Show()
end

--[[
        CreateFrame: Creates the frame for an ability
        arguments:
            id: The id (number) of the ability, between 1 and 12.
        returns:
            option setting in DB
    ]]
function Abilities:CreateFrame(id)

    local frameName = "DHUD4_A"..id
    local frame, frameTexture, frameBorder
    frame = CreateFrame("Button", frameName, self.parent)
    frame:SetHeight(26)
    frame:SetWidth(26)
    frame:SetScale(self.scale)
    frameTexture = frame:CreateTexture(frameName.."Texture", "BORDER");
    frameTexture:SetVertexColor(1, 1, 1);
    frameTexture:SetPoint("CENTER", frame, "CENTER", 0, 0);
    frameTexture:SetHeight(26);
    frameTexture:SetWidth(26);
    frameTexture:SetTexCoord(0, 1, 0, 1);
    frameBorder = frame:CreateTexture(frameName.."Border", "ARTWORK")
    frameBorder:SetTexture(DHUD4:GetCurrentTexture().."auraborder")
    if ( not frameBorder:GetTexture() ) then
        frameBorder:SetTexture("Interface\\AddOns\\DHUD4\\textures\\DHUD\\auraborder")
    end
    frameBorder:SetVertexColor(1, 1, 1)
    frameBorder:SetPoint("CENTER", frame, "CENTER", 0, 0)
    frameBorder:SetHeight(26*1.6)
    frameBorder:SetWidth(26*1.6)
    frameBorder:SetTexCoord(0, 1, 0, 1)
    frameBorder:SetBlendMode("BLEND")
    frameBorder:Show()
    font = frame:CreateFontString(frameName.."Text", "OVERLAY")
    font:SetFontObject(GameFontHighlightSmall)
    font:ClearAllPoints()
    font:SetPoint("CENTER", frame, "CENTER", 0, 0)
    font:SetText("")
    font:SetJustifyH("CENTER")
    font:SetJustifyV("CENTER")
    font:SetWidth(26*2)
    font:Hide()
end

function Abilities:UpdateAuraPosition(frame, id)

    local x, y, yPos, point
	local xCol = 0
    yPos = (-25 + (34 * id))
    y = (-25 + (34 * id))*self.scale
    if ( self.side == "l" ) then
        -- y = 0,0014x^2 - 0,3482x - 20,468
        xDev = - 30 - 6/self.scale
        x = ((0.0014/self.scale)*y^2 - (0.3482/self.scale)*y - (20.468/self.scale))
        point = "LEFT"
    else
        -- y = -0,0014x^2 + 0,3482x + 20,468
        xDev = 30 + 6/self.scale
        x = (-(0.0014/self.scale)*y^2 + (0.3482/self.scale)*y + (20.468/self.scale))
        point = "RIGHT"
    end
    frame:ClearAllPoints()
	frame:SetPoint(point, self.parent, "BOTTOM", x + xDev + xCol, yPos)
end


function Abilities:HideAll()

    --DHUD4:Debug("Abilities:HideAbilities");
    for i = 1, 12 do
        frameName = "DHUD4_A"..i
        frame = _G[frameName]
        if ( frame ) then
            frame:Hide()
            frame:SetScript("OnUpdate", nil)
        end
    end
end

function Abilities:HideRange(from, to)

    --DHUD4:Debug("Abilities:HideAbilities");
    for i = from, to do
        frameName = "DHUD4_A"..i
        frame = _G[frameName]
        if ( frame ) then
            frame:Hide()
            frame:SetScript("OnUpdate", nil)
        end
    end
end

--[[
    ColorizeBorder: Set the color of an ability frame border and it's text
        arguments:
            id: The id (number) of the ability, between 1 and 12.
            text: boolean, also colorize text
]]
function Abilities:Colorize(id)

    --DHUD4:Debug("Abilities:Colorize", id, colors)
    local frameName = "DHUD4_A"..id
    local frame = _G[frameName]
    if ( frame ) then
        local r, g, b, diff
        local percent = frame.left / frame.duration
        local threshold1, threshold2  = 0.6, 0.2
        if ( percent <= threshold2 ) then
            r, g, b = colors.l.r, colors.l.g, colors.l.b
        elseif ( percent <= threshold1) then
            diff = 1 - (percent - threshold2) / (threshold1 - threshold2)
            r = colors.h.r - (colors.h.r - colors.l.r) * diff
            g = colors.h.g - (colors.h.g - colors.l.g) * diff
            b = colors.h.b - (colors.h.b - colors.l.b) * diff
        elseif ( percent < 1) then
            diff = 1 - (percent - threshold1) / (1 - threshold1);
            r = colors.f.r - (colors.f.r - colors.h.r) * diff
            g = colors.f.g - (colors.f.g - colors.h.g) * diff
            b = colors.f.b - (colors.f.b - colors.h.b) * diff
        else
            r, g, b = colors.f.r, colors.f.g, colors.f.b
        end
        --self:Debug("colorize", r, g, b)
        if (r < 0) then r = 0; end
        if (r > 1) then r = 1; end
        if (g < 0) then g = 0; end
        if (g > 1) then g = 1; end
        if (b < 0) then b = 0; end
        if (b > 1) then b = 1; end
        frameBorder = _G[frameName.."Border"]
        frameBorder:SetVertexColor(r, g, b)
        frameText = _G[frameName.."Text"]
        frameText:SetTextColor(r, g, b)
    end
end

function Abilities:DHUD4_BAR(event, bar, visible)

    --DHUD4:Debug("Abilities:DHUD4_BAR", bar, visible)
    visibleBars[bar] = visible
    if ( string_find(bar, self.side) ) then
        self:PlaceAbilitiesFrame()
    end
end


function Abilities:SetBorderColors(c)
    colors = c
end

function Abilities:updateCountTextures(texture, icon, count, filter, tips, duration, expirationTime)

    for i = 1, 5 do
        frameName = "DHUD4_A"..i
        frame = _G[frameName]
        if ( not frame ) then
            self:CreateFrame(i)
            frame = _G[frameName]
        end
        frame:EnableMouse(tips)
        if (i <= count) then
            self:UpdateAuraPosition(frame, i)
            frameTexture = _G[frameName.."Texture"]
            if (texture) then
                frameTexture:SetTexture(DHUD4:GetCurrentTexture().."\\Abilities\\"..texture..i)
                if ( not frameTexture:GetTexture() ) then
                    frameTexture:SetTexture("Interface\\AddOns\\DHUD4\\textures\\DHUD\\Abilities\\"..texture..i)
                end
            else
                frameTexture:SetTexture(icon)
            end
            if (filter) then
                frame.filter = filter
                frame.duration = duration
                frame.hasAura = true;
                frame.auraTimer = 0;
                frame.id = i
                frame.left = expirationTime - GetTime();
                Abilities:Colorize(i)
                frame:SetScript("OnUpdate", OnUpdateAura);
                if ( tips  ) then
                    frame:SetScript("OnEnter", function(self)
                            if not self:IsVisible() or ( self:GetEffectiveAlpha() == 0 ) then
                                return;
                            end
                            GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
                            GameTooltip:SetUnitAura("target", self.id, self.filter);
                        end )
                    frame:SetScript("OnLeave", function()
                            GameTooltip:Hide();
                        end )
                end
            end
            frame:Show()
        else
            frame:Hide()
            frame:SetScript("OnEnter", nil)
            frame:SetScript("OnLeave", nil)
            frame:SetScript("OnUpdate", nil)
        end
    end
end

function Abilities:setTimer(id)

    local frameName = "DHUD4_A"..id
    local frame = _G[frameName]
    if ( frame ) then
        frame.timer = true
        frameText = _G[frameName.."Text"]
        frameText:SetText(DHUD4:FormatTime(frame.left))
        local w = frameText:GetStringWidth() + 10
        frameText:SetWidth(w)
        frameText:Show()
    end
end

function Abilities:UpdateCombos(who)

    --DHUD4:Debug("Abilities:UpdateCombos", who)
    local i, frame, frameName, frameTexture
    local points = GetComboPoints(who, "target")
    --local texture = DHUD4:GetCurrentTexture().."\\Abilities\\c"
    self:updateCountTextures("c", nil, points)
end

function Abilities:UpdateBuff(buff, timer, tips, texture)

    --DHUD4:Debug("Abilities:UpdateBuff", buff)
    local r, g, b, hex, button, timeLeft;
    local name, icon, count, duration, expirationTime, unitCaster;
    name, _, icon, count, _, duration, expirationTime, unitCaster = UnitBuff("target", buff, nil, "PLAYER");
    if ( name ) then
        self:updateCountTextures(texture, icon, count, "HELPFUL", tips, duration, expirationTime)
        if (timer) then
            self:setTimer(1)
        end
    else
        self:HideRange(1, 5)
    end
end

function Abilities:UpdateDebuff(buff, timer, tips, texture)

    --DHUD4:Debug("Abilities:UpdateBuff", buff)
    local r, g, b, hex, button, timeLeft;
    local name, icon, count, duration, expirationTime, unitCaster;
    name, _, icon, count, _, duration, expirationTime, unitCaster = UnitDebuff("target", buff, nil, "PLAYER");
    if ( name ) then
        self:updateCountTextures(texture, icon, count, "HARMFUL", tips, duration, expirationTime)
        if (timer) then
            self:setTimer(1)
        end
    else
        self:HideRange(1, 5)
    end
end

function Abilities:_SetLayout(count, showText)

    --DHUD4:Debug("Abilities:_SetLayout", count, showText)
    for i = 1, count do
        local frameName = "DHUD4_A"..i
        local frame = _G[frameName]
        if ( not frame ) then
            self:CreateFrame(i)
            frame = _G[frameName]
        end
        -- set the icon
        local frameTexture = _G[frameName.."Texture"]
        frameTexture:SetTexture("Interface\\AddOns\\DHUD4\\icon")
        if (showText) then
            -- set the text
            local frameCooldown = _G[frameName.."Text"]
            frameCooldown:SetText(j)
        end
        self:UpdateAuraPosition(frame, i)
        frame:Show()
    end
end